import { BaseGameDataMessage } from "@skeldjs/protocol";
import { CustomNetworkTransform, CustomNetworkTransformEvents, PlayerControl, PlayerControlEvents, PlayerPhysics, PlayerPhysicsEvents } from "./objects";
import { Hostable } from "./Hostable";
import { PlayerJoinEvent, PlayerLeaveEvent, PlayerReadyEvent, PlayerSceneChangeEvent, PlayerSetHostEvent, PlayerSpawnEvent } from "./events";
import { BasicEvent, EventEmitter, ExtractEventTypes } from "@skeldjs/events";
import { NetworkableEvents } from "./Networkable";
export declare type PlayerDataEvents<RoomType extends Hostable = Hostable> = NetworkableEvents<RoomType> & PlayerControlEvents<RoomType> & PlayerPhysicsEvents<RoomType> & CustomNetworkTransformEvents<RoomType> & ExtractEventTypes<[
    PlayerReadyEvent<RoomType>,
    PlayerJoinEvent<RoomType>,
    PlayerLeaveEvent<RoomType>,
    PlayerSetHostEvent<RoomType>,
    PlayerSceneChangeEvent<RoomType>,
    PlayerSpawnEvent<RoomType>
]>;
/**
 * Represents the player of a client connected to the room.
 *
 * See {@link PlayerDataEvents} for events to listen to.
 */
export declare class PlayerData<RoomType extends Hostable = Hostable> extends EventEmitter<PlayerDataEvents<RoomType>> {
    /**
     * The room that this player object belongs to.
     */
    room: RoomType;
    /**
     * This player's server-unique client ID.
     */
    clientId: number;
    /**
     * Whether or not this player is readied up to start the game.
     */
    isReady: boolean;
    /**
     * Whether or not this player is in the game scene.
     */
    inScene: boolean;
    /**
     * Whether or not this player has left the game.
     */
    left: boolean;
    /**
     * The message stream to be sent on fixed update.
     */
    stream: BaseGameDataMessage[];
    character: PlayerControl<RoomType> | undefined;
    constructor(room: RoomType, clientId: number);
    emit<Event extends BasicEvent>(event: Event): Promise<Event>;
    /**
     * The player's control component.
     */
    get control(): PlayerControl<RoomType> | undefined;
    /**
     * The player's physics component.
     */
    get physics(): PlayerPhysics<RoomType> | undefined;
    /**
     * The player's movement component.
     */
    get transform(): CustomNetworkTransform<RoomType> | undefined;
    get isFakePlayer(): boolean | undefined;
    /**
     * The player's information.
     */
    get info(): import(".").PlayerInfo<RoomType> | undefined;
    /**
     * The room-unique player ID of the player.
     */
    get playerId(): number | undefined;
    /**
     * Whether or not the player has fully spawned.
     */
    get hasSpawned(): boolean;
    /**
     * Whether or not the player is the host of the room they belong in.
     */
    get isHost(): boolean;
    /**
     * Whether or not the player is the current client's player.
     */
    get isMe(): boolean;
    /**
     * Mark as readied up to start the game.
     */
    ready(): Promise<void>;
    /**
     * Despawn all components on the player,
     */
    despawn(): void;
}
