"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MeetingHud = void 0;
const util_1 = require("@skeldjs/util");
const constant_1 = require("@skeldjs/constant");
const protocol_1 = require("@skeldjs/protocol");
const Networkable_1 = require("../Networkable");
const PlayerVoteState_1 = require("../misc/PlayerVoteState");
const PlayerVoteArea_1 = require("../misc/PlayerVoteArea");
const events_1 = require("../events");
class MeetingHud extends Networkable_1.Networkable {
    constructor(room, spawnType, netid, ownerid, flags, data) {
        super(room, spawnType, netid, ownerid, flags, data);
        this.dirtyBit || (this.dirtyBit = 0);
        this.voteStates || (this.voteStates = new Map);
    }
    Awake() {
        if (this.room.gameData) {
            this.voteStates = new Map([...this.room.gameData.players]
                .filter(([, player]) => player.playerId !== undefined)
                .map(([, player]) => {
                return [
                    player.playerId,
                    new PlayerVoteArea_1.PlayerVoteArea(this, player.playerId, PlayerVoteState_1.VoteStateSpecialId.NotVoted, false),
                ];
            }));
        }
        this.ranOutOfTimeTimeout = setTimeout(() => {
            for (const [, voteState] of this.voteStates) {
                if (voteState.votedForId === 255) {
                    voteState.setMissed();
                }
            }
            this.checkForVoteComplete(true);
        }, 8000 + this.room.settings.discussionTime * 1000 + this.room.settings.votingTime * 1000);
    }
    getComponent(component) {
        if (component === MeetingHud) {
            return this.components[0];
        }
        return super.getComponent(component);
    }
    get owner() {
        return super.owner;
    }
    Deserialize(reader, spawn = false) {
        if (spawn) {
            this.dirtyBit = 0;
            this.voteStates = new Map;
        }
        const numVotes = reader.packed();
        for (let i = 0; i < numVotes; i++) {
            const [playerId, mreader] = reader.message();
            const player = this.room.getPlayerByPlayerId(playerId);
            if (!player)
                continue;
            const oldState = this.voteStates.get(playerId);
            const newState = PlayerVoteArea_1.PlayerVoteArea.Deserialize(mreader, this, playerId);
            this.voteStates.set(playerId, newState);
            if (!(oldState === null || oldState === void 0 ? void 0 : oldState.hasVoted) && newState.hasVoted) {
                this.emit(new events_1.MeetingHudVoteCastEvent(this.room, this, undefined, player, newState.votedFor));
            }
            else if ((oldState === null || oldState === void 0 ? void 0 : oldState.votedFor) && !newState.hasVoted) {
                this.emit(new events_1.MeetingHudClearVoteEvent(this.room, this, undefined, player));
            }
        }
    }
    Serialize(writer, spawn = false) {
        writer.upacked(this.voteStates.size);
        for (const [, state] of this.voteStates) {
            writer.begin(state.playerId);
            state.Serialize(writer);
            writer.end();
        }
        this.dirtyBit = 0;
        return true;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.Close:
                await this._handleClose(rpc);
                break;
            case constant_1.RpcMessageTag.VotingComplete:
                await this._handleVotingComplete(rpc);
                break;
            case constant_1.RpcMessageTag.CastVote:
                await this._handleCastVote(rpc);
                break;
            case constant_1.RpcMessageTag.ClearVote:
                await this._handleClearVote(rpc);
                break;
        }
    }
    async _handleClose(rpc) {
        await this.emit(new events_1.MeetingHudCloseEvent(this.room, this, rpc));
    }
    _close() {
        this.room["_despawnComponent"](this);
    }
    _rpcClose() {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.CloseMessage));
    }
    /**
     * Close the meeting hud for all clients. This is a host-only operation on
     * official servers.
     */
    close() {
        this._close();
        this._rpcClose();
    }
    checkForVoteComplete(isTimeout) {
        const states = [...this.voteStates];
        if (states.every(([, state]) => state.hasVoted || !state.canVote) || isTimeout) {
            let tie = false;
            let exiled;
            let exiledVotes = 0;
            let numSkips = 0;
            for (const [, state2] of states) {
                if (state2.votedForId === PlayerVoteState_1.VoteStateSpecialId.SkippedVote) {
                    numSkips++;
                }
            }
            for (const [, state] of states) {
                let num = 0;
                for (const [, state2] of states) {
                    if (state2.votedForId === state.playerId) {
                        num++;
                    }
                }
                if (num) {
                    if (num > exiledVotes) {
                        tie = false;
                        exiled = state.player;
                        exiledVotes = num;
                    }
                    else if (num === exiledVotes) {
                        tie = true;
                        exiled = undefined;
                    }
                }
            }
            if (numSkips >= exiledVotes) {
                exiled = undefined;
            }
            this.votingComplete(tie, exiled);
            util_1.sleep(5000).then(() => {
                var _a;
                if ((_a = this.exiled) === null || _a === void 0 ? void 0 : _a.info) {
                    this.exiled.info.flags |= constant_1.PlayerDataFlags.IsDead;
                }
                this.close();
            });
        }
    }
    async _handleCastVote(rpc) {
        var _a, _b;
        const voter = this.voteStates.get(rpc.votingid);
        const player = this.room.getPlayerByPlayerId(rpc.votingid);
        const suspect = rpc.suspectid === 0xfd
            ? undefined
            : this.room.getPlayerByPlayerId(rpc.suspectid);
        if (this.room.hostIsMe && player && voter && (suspect || rpc.suspectid === 0xfd)) {
            this._castVote(voter, suspect);
            const ev = await this.emit(new events_1.MeetingHudVoteCastEvent(this.room, this, rpc, player, suspect));
            if (ev.reverted) {
                if (player) {
                    await this.clearVote(player);
                }
            }
            else {
                (_b = (_a = this.room.host) === null || _a === void 0 ? void 0 : _a.control) === null || _b === void 0 ? void 0 : _b.sendChatNote(player, constant_1.ChatNoteType.DidVote);
                this.checkForVoteComplete(false);
            }
        }
    }
    _castVote(voterState, suspect) {
        if (voterState) {
            if (suspect) {
                voterState.setSuspect(suspect.playerId);
            }
            else {
                voterState.setSkipped();
            }
        }
    }
    async _rpcCastVote(voter, suspect) {
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.CastVoteMessage(voter.playerId, suspect
                ? suspect.playerId
                : 255))
        ], true, this.room.hostId);
    }
    /**
     * Cast a vote on behalf of a user (or yourself). Casting for another player
     * other than the player calling the function is a host-only operation on
     * official servers.
     * @param voter The player who is voting.
     * @param suspect The player to vote for.
     * @example
     *```typescript
     * // Make everyone vote a certain player.
     * for ([ clientid, player ] of room.players) {
     *   if (player !== suspect) {
     *     room.meetinghud.castVote(player, suspect);
     *   }
     * }
     * ```
     */
    async castVote(voter, suspect) {
        var _a, _b;
        const player = this.room.resolvePlayer(voter);
        const _suspect = suspect === "skip" ? undefined : this.room.resolvePlayer(suspect);
        if (player && player.playerId !== undefined) {
            const votingState = this.voteStates.get(player.playerId);
            if (votingState) {
                if (this.room.hostIsMe) {
                    this._castVote(votingState, _suspect);
                    const ev = await this.emit(new events_1.MeetingHudVoteCastEvent(this.room, this, undefined, player, _suspect));
                    if (ev.reverted) {
                        this._clearVote(votingState);
                    }
                    else {
                        (_b = (_a = this.room.host) === null || _a === void 0 ? void 0 : _a.control) === null || _b === void 0 ? void 0 : _b.sendChatNote(player, constant_1.ChatNoteType.DidVote);
                        this.checkForVoteComplete(false);
                    }
                }
                else {
                    await this._rpcCastVote(player, _suspect);
                }
            }
        }
    }
    async _handleClearVote(rpc) {
        void rpc;
        const player = this.room.myPlayer;
        if (player && player.playerId !== undefined) {
            const voter = this.voteStates.get(player.playerId);
            if (voter === null || voter === void 0 ? void 0 : voter.hasVoted) {
                this._clearVote(voter);
                await this.emit(new events_1.MeetingHudClearVoteEvent(this.room, this, rpc, voter.player));
            }
        }
    }
    _clearVote(voter) {
        if (voter.hasVoted) {
            voter.clearVote();
            voter.dirty = true;
        }
    }
    async _rpcClearVote(voter) {
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.ClearVoteMessage),
        ], true, this.room.getPlayerByPlayerId(voter.playerId));
    }
    /**
     * Remove someone's vote (usually due to the player they voted for getting disconnected).
     * This is a host-only operation on official servers.
     * @param resolvable The player to remove the vote of.
     */
    async clearVote(voter) {
        const player = this.room.resolvePlayer(voter);
        if (player && player.playerId !== undefined) {
            const _voter = this.voteStates.get(player.playerId);
            if (_voter) {
                this._clearVote(_voter);
                await this.emit(new events_1.MeetingHudClearVoteEvent(this.room, this, undefined, _voter.player));
                await this._rpcClearVote(_voter);
            }
        }
    }
    async _handleVotingComplete(rpc) {
        const exiled = rpc.exiledid === 0xff
            ? undefined
            : this.room.getPlayerByPlayerId(rpc.exiledid);
        const playerStates = rpc.states.map((state, i) => {
            return new PlayerVoteState_1.PlayerVoteState(this.room, state.playerId, state.votedForId);
        });
        this._votingComplete(playerStates, rpc.tie, exiled);
        await this.emit(new events_1.MeetingHudVotingCompleteEvent(this.room, this, rpc, rpc.tie, new Map(playerStates.map(state => [state.playerId, state])), exiled));
    }
    _votingComplete(states, tie, exiled) {
        for (let i = 0; i < states.length; i++) {
            const state = this.voteStates.get(i);
            if (state) {
                state.votedForId = states[i].votedForId;
            }
        }
        this.tie = tie;
        this.exiled = exiled;
        if (this.ranOutOfTimeTimeout)
            clearInterval(this.ranOutOfTimeTimeout);
    }
    _rpcVotingComplete(states, tie, exiled) {
        var _a;
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.VotingCompleteMessage(states.map(state => new protocol_1.VoteState(state.playerId, state.votedForId)), exiled ? (_a = exiled.playerId) !== null && _a !== void 0 ? _a : 0xff : 0xff, tie)));
    }
    /**
     * End the meeting with specified results. This is a host-only operation
     * on official servers.
     * @param tie Whether this meeting resulted in a tie of votes.
     * @param exiled The player that was ejected, if any.
     */
    votingComplete(tie = false, exiled) {
        const _exiled = exiled ? this.room.resolvePlayer(exiled) : undefined;
        const voteStates = new Array(this.room.gameData.players.size);
        let i = 0;
        for (const [playerId, state] of this.voteStates) {
            voteStates[i] = new PlayerVoteState_1.PlayerVoteState(this.room, playerId, state.votedForId);
            i++;
        }
        this._votingComplete(voteStates, tie, _exiled);
        this._rpcVotingComplete(voteStates, tie, _exiled);
        this.emit(new events_1.MeetingHudVotingCompleteEvent(this.room, this, undefined, tie, new Map(voteStates.map(vote => [vote.playerId, vote])), _exiled));
    }
}
exports.MeetingHud = MeetingHud;
//# sourceMappingURL=MeetingHud.js.map